@@ -17,6 +17,14 @@ import termcolor
 import random
 import py_compile
 from pathlib import Path
+from clouderizer.cldz_aws_lambda import lambda_deploy, lambda_delete, aws_region_set
+from clouderizer.utils import check_aws_cli_exists, check_sam_cli_exists, prepend_line
+from yaml import load, dump
+# from clouderizer.utils import prepend_line
+try:
+    from yaml import CLoader as Loader, CDumper as Dumper
+except ImportError:
+    from yaml import Loader, Dumper
 
 BASE_URL="https://showcase.clouderizer.com"
 SYNC_URL="https://serverless.clouderizer.com/function/"
@@ -25,8 +33,28 @@ upload_endpoint = BASE_URL + '/api/awsconfig/generatepresignedurl'
 
 LOCAL_PATH=os.getenv("HOME") + "/.clouderizer/creds"
 LOCAL_DIR=os.getenv("HOME") + "/.clouderizer"
+DOCKERFILES_LOC = LOCAL_DIR+"/dockerfiles/"
+LAMBDA_FUNCTION_DIR = LOCAL_DIR+ "/lambda-dir/function/"
 
 Path(LOCAL_DIR).mkdir(parents=True, exist_ok=True)
+
+if "beta" in BASE_URL:
+	Path(LOCAL_DIR+"/beta").mkdir(parents=True,exist_ok=True)
+	LOCAL_PATH = os.getenv("HOME") + "/.clouderizer/beta/creds"
+
+dockerfiles = ['AutomlDockerfile','PythonDockerfile']
+for dockerfile in dockerfiles:
+	if not os.path.exists(DOCKERFILES_LOC):
+		Path(DOCKERFILES_LOC).mkdir(parents=True,exist_ok=True)
+
+	if not os.path.exists(DOCKERFILES_LOC+dockerfile):
+		resp=requests.post(BASE_URL+'/givemelambdadockerfile',json={
+			"type": dockerfile
+		},headers={"Content-Type":"application/json"})
+
+		if resp.status_code==200:
+			open(DOCKERFILES_LOC+dockerfile,"w").write(resp.text)
+
 display_table = PrettyTable()
 display_table.field_names = ['Project Name','Created At', 'Status', 'Status Message']
 
@@ -34,6 +62,7 @@ status_truncation=30
 SUPPORTED_MODEL_EXECUTION_TYPES=['h2o','dai','pmml','python','onnx']
 SUPPORTED_NOTEBOOK_EXECUTION_TYPES=['python']
 SUPPORTED_INFRA_TYPES=['standard','highmemory','gpu']
+LAMBDA_SUPPORTED_INFRA_TYPES=['standard','highmemory']
 SUPPORTED_IMAGE_TYPES=['standard','torch','tensorflow']
 
 SCOPES=['https://www.googleapis.com/auth/userinfo.profile','https://www.googleapis.com/auth/userinfo.email' ,'openid']
@@ -50,6 +79,12 @@ def signal_handler(sig, frame):
 signal.signal(signal.SIGINT, signal_handler)
 stop_loading_thread=False
 
+def display_timer(msg, seconds):
+	for i in range(seconds):
+		sys.stdout.write("\r{} {} seconds ".format(msg, seconds-i))
+		sleep(1)
+		sys.stdout.flush()
+
 def colorful(text,color=None):
 	if color:
 		return termcolor.colored(text,color=color,attrs=['bold'])
@@ -117,7 +152,59 @@ def checkFileType(filename,format):
 	print("Incorrect format for "+filename + " expected extension "+format)
 	exit(0)
 
+def parse_yaml(schemaPath):
 
+	if not os.path.isfile(schemaPath):
+		print("schema file not found in specified location")
+		exit(0)
+
+	schemaFile = open(schemaPath).read()
+
+	schemaJson = load(schemaFile,Loader=Loader)
+
+	# if 'input' not in schemaJson or 'output' not in schemaJson:
+	# 	print("schema format not according to specifications")
+	# 	exit(0)
+
+	inputList = []
+	if 'input' in schemaJson:
+		inputList = schemaJson['input']
+
+		for input in inputList:
+			if "allowedValues" not in input:
+				input["allowedValues"] = [] 
+
+	outputList = []
+	if 'output' in schemaJson:
+		outputList = schemaJson['output']
+
+		for output in outputList:
+			if "allowedValues" not in output:
+				output["allowedValues"] = []
+
+	inputAttrs =  [
+		{
+			"type" : "Input Columns",
+			"image" : "Grid_Table_1",
+			"format" : "csv",
+			"formats" : [],
+			"inputList" : inputList,
+			"rawinputList" : inputList
+		}
+	]
+
+	outputAttrs = [
+		{
+			"type" : "Output Columns",
+			"image" : "Grid_Table_1",
+			"format" : "csv",
+			"formats" : [],
+			"outputList" : outputList,
+			"finaloutput" : outputList
+		}
+	]
+
+	return inputAttrs,outputAttrs
 ################ pipreqsnb code ################
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
@@ -292,6 +379,17 @@ def upload_to_showcase(file_name,file_loc,company,auth_token):
 			# },headers={"Content-Type":"application/json"})
 	return "Some error occurred while uploading... try again"
 
+def update_serving_model(servingmodel,data,auth_token):
+	# print(servingmodel,data)
+	try:
+
+		auth_headers={"Authorization": "Bearer "+auth_token}
+		resp = requests.patch(BASE_URL+'/api/servingmodel/'+servingmodel,data=data,headers=auth_headers)
+		# print(resp)
+		# print(resp.text)
+		return
+	except Exception as e:
+		pass
 # def call_api(url, call_type, headers=None, data=None):
 #   try: 
 #     if(call_type=='get'):
@@ -320,39 +418,48 @@ def createEvent(action,company,auth_token,projectId=None,projectName=None):
 	except Exception as e:
 		pass
 
-def create_serving_project(filename,companyid,auth_token,retry=0):
-	name=unique_project_name(filename)
-	print("Creating a project named "+name)
+def create_serving_project(filename,companyid,auth_token,projectName=None,inputAttrs=None,outputAttrs=None,retry=0):
+	if not projectName:
+		name=unique_project_name(filename)
+	else:
+		name=projectName
+	print("\nCreating a project named "+name)
 
-	input_attr = [ 
-		{
-			"type" : "Input Columns",
-			"image" : "Grid_Table_1",
-			"format" : "csv",
-			"formats" : [],
-			"inputList" : [],
-			"rawinputList" : [],
-			"subtype" : ""
-		}
-	]
-
-	output_attr = [
-		{
-			"type" : "Output Columns",
-			"image" : "Grid_Table_1",
-			"format" : "csv",
-			"formats" : [],
-			"outputList" : [],
-			"subtype" : ""
-		}
-	]
+	if not inputAttrs:
+		input_attr = [ 
+			{
+				"type" : "Input Columns",
+				"image" : "Grid_Table_1",
+				"format" : "csv",
+				"formats" : [],
+				"inputList" : [],
+				"rawinputList" : [],
+				"subtype" : ""
+			}
+		]
+	else:
+		input_attr = inputAttrs
+	
+	if not outputAttrs:
+		output_attr = [
+			{
+				"type" : "Output Columns",
+				"image" : "Grid_Table_1",
+				"format" : "csv",
+				"formats" : [],
+				"outputList" : [],
+				"subtype" : ""
+			}
+		]
+	else:
+		output_attr = outputAttrs
 
 	auth_headers={"Authorization": "Bearer "+auth_token}
 	project = {
 		"name": name,
 		"company": companyid,
-		# "inputAttr": input_attr,
-		# "outputAttr": output_attr
+		"inputAttr": json.dumps(input_attr),
+		"outputAttr": json.dumps(output_attr)
 		# "inputAttr": [{"type":"Input Columns", "image": "Grid_Table_1", "format": "csv", "formats": [], "inputList":[], "rawinputList":[], "subtype":""}], 
 		# "outputAttr": [{"type":"Output Columns", "image": "Grid_Table_1", "format": "csv", "formats": [], "outputList":[], "subtype":""}]
 		# "company": "ad8b00cf-956f-4202-9bc8-7cf8c5eb5ff2"
@@ -371,7 +478,8 @@ def create_serving_project(filename,companyid,auth_token,retry=0):
 		return create_serving_project(filename,companyid,auth_token,retry)
 
 
-def create_serving_model(file_name,s3Path,servingproject,servingprojectid,pipPackages,auth_token,training,model_type,subtype,image_type,preprocess=None,postprocess=None,predict=None,retry=0):
+def create_serving_model(file_name,s3Path,servingproject,servingprojectid,pipPackages,auth_token,training,model_type,subtype,image_type,preprocess=None,postprocess=None,predict=None,inputAttrs=[],outputAttrs=[],retry=0):
+	# print("Inside")
 	company=servingproject["company"]
 	auth_headers={"Authorization": "Bearer "+auth_token}
 
@@ -420,7 +528,9 @@ def create_serving_model(file_name,s3Path,servingproject,servingprojectid,pipPac
 		"postprocessCode": postprocessCode,
 		"predictCode": predictCode,
 		"predictCodePath": predictS3Path,
-		"imagetype" : image_type
+		"imagetype" : image_type,
+		"inputAttr" : inputAttrs,
+		"outputAttr": outputAttrs
 	}
 
 	model_resp=requests.post(BASE_URL+'/api/servingmodel',data=model,headers=auth_headers)
@@ -437,6 +547,13 @@ def create_serving_model(file_name,s3Path,servingproject,servingprojectid,pipPac
 		sleep(5*retry)
 		return create_serving_model(file_name,s3Path,servingproject,servingprojectid,pipPackages,auth_token,training,model_type,subtype,image_type,preprocess,postprocess,predict,retry)
 
+def fetchprojectconfig(servingproject):
+	# print("Serving project", servingproject)
+	response = requests.post(BASE_URL+'/api/servingproject/fetchprojectconfig',data={"servingproject":servingproject})
+
+	if response.status_code==200:
+		return parse_json(response.text)
+
 def parser(model_path,servingproject,servingmodelid,subtype,auth_token):
 	file_name=model_path.split("/")[-1]
 	s3Path=servingproject["company"]+"/"+servingproject["name"]+"/"+file_name
@@ -650,6 +767,10 @@ def find_full_project_name(name):
 	user_data=f.read()
 	f.close()
 	user_data = parse_json(user_data)
+
+	if 'projects' not in user_data:
+		print("Some error occurred, check your projects using cldz ls")
+		exit(0)
 	for project_name in user_data['projects']:
 		if project_name == name:
 			return name
@@ -663,6 +784,10 @@ def find_full_project_name(name):
 			fullProjectName=project_name
 			count+=1
 
+	if len(matchingProjectsArr) == 0:
+		print("No project name found!")
+		exit(0)
+
 	if len(matchingProjectsArr)==1:
 		return fullProjectName
 
@@ -675,10 +800,14 @@ def find_full_project_name(name):
 	
 	return name
 
-def invoke_publishproject(servingprojectid, infra, auth_token):
+def invoke_publishproject(servingprojectid, infra, auth_token, invokepublishproject=True):
 	# print(servingprojectid)
+	invoke_url = BASE_URL+'/api/servingproject/publishproject'
+	if not invokepublishproject:
+		invoke_url = BASE_URL+'/api/servingproject/publishprojectlambda'
+
 	auth_headers={"Authorization": "Bearer "+auth_token}
-	publishproject=requests.post(BASE_URL+'/api/servingproject/publishproject',headers=auth_headers,data={"projectId":servingprojectid,"infratype":infra})
+	publishproject=requests.post(invoke_url,headers=auth_headers,data={"projectId":servingprojectid,"infratype":infra})
 	
 	if publishproject.status_code==401:
 		return "Authentication was unsuccessful! try cldz login"
@@ -706,6 +835,7 @@ def wait_for_saved_successfully(servingprojectid,auth_token,retry=0):
 	headers = {"Authorization":"Bearer "+auth_token}
 	published_serving_projects=requests.get(BASE_URL+'/api/publishedservingproject?where={"servingproject":"%s"}' % servingprojectid,headers=headers)
 
+	# print(published_serving_projects.text)
 	published_serving_projects=parse_json(published_serving_projects.text)
 
 	if len(published_serving_projects)==0:
@@ -717,6 +847,7 @@ def wait_for_saved_successfully(servingprojectid,auth_token,retry=0):
 		return wait_for_saved_successfully(servingprojectid,auth_token,retry+1)
 
 	status_message=published_serving_projects[0]["status_message"]
+	# print(status_message)
 	if "Saved" in status_message or "Deploy" in status_message or "success" in status_message:
 		loadingDisplay=False
 		stop_loading_thread=True
@@ -758,20 +889,30 @@ class Clouderizer(object):
 	def __init__(self):
 		pass
 
-	def login(self):
-		print("Follow this url and get an api key: "+ BASE_URL +"/api/auth/google?source=cli")
+	def login(self,token=None):
+		if not token:
+			print("Follow this url and get an authentication token: "+ BASE_URL +"/api/auth/google?source=cli")
+			webbrowser.open(BASE_URL+'/api/auth/google?source=cli', new=1, autoraise=True)
+			apiKey= input("Paste your authentication token:")
 
-		webbrowser.open(BASE_URL+'/api/auth/google?source=cli', new=1, autoraise=True)
-		apiKey= input("Paste your api key:")
+			try:
+				payload=jwt.decode(apiKey,'oursecret', algorithms=['HS256'])
+			except Exception as e:
+				print("Could not verify token. Is it correct?")
+				return
 
-		try:
-			payload=jwt.decode(apiKey,'oursecret', algorithms=['HS256'])
-		except Exception as e:
-			print("Could not verify token. Is it correct?")
-			return
+			payload["token"]=apiKey
+			headers = {"Authorization":"Bearer "+payload["token"]}
 
-		payload["token"]=apiKey
-		headers = {"Authorization":"Bearer "+payload["token"]}
+		else:
+			try:
+				payload=jwt.decode(token,'oursecret', algorithms=['HS256'])
+			except Exception as e:
+				print("token verification failed. Is it correct?")
+				return
+
+			payload["token"]=token
+			headers = {"Authorization":"Bearer "+payload["token"]}
 
 		# get company dets		
 		customer_dets=requests.get(BASE_URL+'/api/customer/'+payload['cid'],headers=headers)
@@ -787,6 +928,8 @@ class Clouderizer(object):
 		if "name" in user_dets:
 			payload["User"]=user_dets["name"]
 
+		payload['projects']=[]
+
 		with open(LOCAL_PATH,'w') as f:
 			f.write(json.dumps(payload))
 
@@ -811,6 +954,7 @@ class Clouderizer(object):
 
 		published_serving_projects=requests.post(BASE_URL+'/api/servingproject/getprojects',data={"company":creds["cid"]})
 
+		# print(published_serving_projects.text)
 		published_serving_projects=parse_json(published_serving_projects.text)
 		clear_local_project_list()
 		# except Exception as e:
@@ -1018,6 +1162,10 @@ class Clouderizer(object):
 		#   print("Could not fetch your details")
 		#   return
 
+		if "name" not in customer_dets:
+			print("Could not fetch project details")
+			return
+
 		published_serving_projects=requests.get(BASE_URL+'/api/publishedservingproject?where={"name":"%s"}' % (customer_dets["name"].lower().replace(" ","")+"-"+project_name.lower().replace("_","-")),headers=headers)
 		# try:
 		published_serving_project = parse_json(published_serving_projects.text)
@@ -1066,15 +1214,25 @@ class Clouderizer(object):
 				print(published_serving_project[0]["status_message"])
 			if "Deployed successfully"==status_message:
 				if project_type=="NOTEBOOK" or not project_type:
-					url=BASE_URL + "/api/async-function/"+published_serving_project[0]["name"]+"/notebook"
-					surl=SYNC_URL + published_serving_project[0]["name"]+"/notebook"
-					print("\nNOTEBOOK ASYNC URL : "+ url)
-					print("\nNOTEBOOK SYNC URL : "+ surl)
-					print("\nSAMPLE ASYNC CURL COMMAND : "+ 'curl -i -X POST '+url + ' -F "param1=x" -F "param2=y" -F "file1=@<filepath1>" -F "file2=@<filepath2>" -H "X-Callback-Url: https://<callback-url-here>"')
-					print("\nSAMPLE SYNC CURL COMMAND : "+ 'curl -i -X POST '+surl + ' -F "param1=x" -F "param2=y" -F "file1=@<filepath1>" -F "file2=@<filepath2>"')
+					if "accessUrl" in serving_model:
+						url = serving_model['accessUrl']
+						print("\nNOTEBOOK URL : "+ url)
+						print("\nSAMPLE ASYNC CURL COMMAND : "+ 'curl -i -X POST '+url + ' -F "param1=x" -F "param2=y" -F "file1=@<filepath1>" -F "file2=@<filepath2>" -H "x-callback-url: https://<callback-url-here>"')
+						print("\nSAMPLE SYNC CURL COMMAND : "+ 'curl -i -X POST '+surl + ' -F "param1=x" -F "param2=y" -F "file1=@<filepath1>" -F "file2=@<filepath2>"')
+					else:
+						url=BASE_URL + "/api/async-function/"+published_serving_project[0]["name"]+"/notebook"
+						surl=SYNC_URL + published_serving_project[0]["name"]+"/notebook"
+						print("\nNOTEBOOK ASYNC URL : "+ url)
+						print("\nNOTEBOOK SYNC URL : "+ surl)
+						print("\nSAMPLE ASYNC CURL COMMAND : "+ 'curl -i -X POST '+url + ' -F "param1=x" -F "param2=y" -F "file1=@<filepath1>" -F "file2=@<filepath2>" -H "X-Callback-Url: https://<callback-url-here>"')
+						print("\nSAMPLE SYNC CURL COMMAND : "+ 'curl -i -X POST '+surl + ' -F "param1=x" -F "param2=y" -F "file1=@<filepath1>" -F "file2=@<filepath2>"')
 				else:
-					surl=SYNC_URL + published_serving_project[0]["name"]+"/predict"
+					if "accessUrl" in serving_model:
+						surl = serving_model["accessUrl"] + '/predict'
+					else:
+						surl=SYNC_URL + published_serving_project[0]["name"]+"/predict"
 					print("\nMODEL URL : "+ surl)
+					print("\nWEB APP URL : " + SYNC_URL+published_serving_project[0]["name"])
 					
 			else:
 				if project_type=="NOTEBOOK":
@@ -1348,7 +1506,7 @@ class Clouderizer(object):
 				# print(updated_model.text)
 			print("Model updated successfully!")
 
-	def deploy(self,file_path=None,requirements=None,predict=None,preprocess=None,postprocess=None,model_type=None,notebook_type=None,infra="standard",image="standard"):
+	def deploy(self,file_path=None,requirements=None,predict=None,preprocess=None,postprocess=None,model_type=None,notebook_type=None,infra="standard",image="standard",schema=None):
 	# def deploy(self,first_path=None,second_path=None,third_path=None,model_type=None,notebook_type=None,enable_gpu=None):
 		global loadingDisplay
 		creds=self.loggedin()
@@ -1368,6 +1526,12 @@ class Clouderizer(object):
 		execution_type="notebook"
 		subtype="pythonscore"
 
+
+		inputAttrs={"inputList":[], "rawinputList":[]} 
+		outputAttrs={"outputlist": [],"finaloutput":[]}
+		if schema:
+			inputAttrs, outputAttrs = parse_yaml(schema)
+
 		if model_type and notebook_type:
 			print("Parameters unclear, both notebook and model types selected")
 			return
@@ -1427,7 +1591,10 @@ class Clouderizer(object):
 
 			print("TYPE : NOTEBOOK")
 			print("NOTEBOOK FILE PATH : " + notebook_path)
-			print("FETCHED PIP PACKAGES : ", pipPackages)			
+			print("FETCHED PIP PACKAGES : ", pipPackages)		
+
+			print("Please check the above project attributes.")
+			display_timer("In case something looks wrong, press Ctrl + C and try again with correct values. Deployment will continue in ",10)	
 
 			servingproject=create_serving_project(notebook_path.split("/")[-1],creds['cid'],auth_token)
 			# print(type(servingproject))
@@ -1439,10 +1606,12 @@ class Clouderizer(object):
 				uploadmodel_resp=upload_to_showcase(s3Path,notebook_path,servingproject["company"],auth_token)
 				if uploadmodel_resp=="Success":
 					try:
-						servingmodel=create_serving_model(file_name,s3Path,servingproject,servingproject["id"],pipPackages,auth_token,"true","regular",subtype,image)
-					except Exception:
+						servingmodel=create_serving_model(file_name,s3Path,servingproject,servingproject["id"],pipPackages,auth_token,"true","regular",subtype,image,inputAttrs=inputAttrs,outputAttrs=outputAttrs)
+					except Exception as e:
+						print(e)
 						print("Some exception occurred while preparing your notebook, please try again!")
 						return
+
 					if type(servingmodel)==dict and "id" in servingmodel:    
 						print("Project created!")
 						publishproject=invoke_publishproject(servingproject["id"],infra,auth_token)
@@ -1463,7 +1632,7 @@ class Clouderizer(object):
 							else:
 								print("Error deploying your project, deploy again")
 								createEvent("deploy failed",creds['cid'],auth_token,servingproject["id"],servingproject["name"])
-								self.delete(servingproject["name"])
+								# self.delete(servingproject["name"])
 								return
 						else:
 							self.delete(servingproject["name"])
@@ -1527,6 +1696,8 @@ class Clouderizer(object):
 			if predict_path:
 				checkPathExists(predict_path)
 				checkFileSyntax(predict_path)
+
+				prepend_line(predict_path,"modelPath='/home/app/function/asset/model.file'")
 				# if not os.path.exists(predict_path):
 				# 	print("Preprocess path does not exist " + predict_path)
 				# 	return	
@@ -1548,7 +1719,10 @@ class Clouderizer(object):
 			print("POSTPROCESS PATH : " ,postprocess_path)
 			print("FETCHED PIP PACKAGES : ", pipPackages)
 
-			servingproject=create_serving_project(model_path.split("/")[-1],creds['cid'],auth_token)
+			print("Please check the above project attributes.")
+			display_timer("In case something looks wrong, press Ctrl + C and try again with correct values. Deployment will continue in ",10)
+
+			servingproject=create_serving_project(model_path.split("/")[-1],creds['cid'],auth_token,inputAttrs=inputAttrs,outputAttrs=outputAttrs)
 			# print(type(servingproject))
 			if type(servingproject)==dict and "id" in servingproject:
 				update_local_project_list(servingproject["name"],"add")
@@ -1595,7 +1769,8 @@ class Clouderizer(object):
 						if subtype in ["python","onnx","pythonscore"]:
 							deployment_model_type="regular"
 						servingmodel=create_serving_model(file_name,s3Path,servingproject,servingproject["id"],pipPackages,auth_token,"false",deployment_model_type,subtype,image,preprocess=preprocess_path,postprocess=postprocess_path,predict=predict_path)
-					except Exception:
+					except Exception as e:
+						print(e)
 						print("Some exception occurred while preparing your model, please try again!")						
 						return
 
@@ -1625,7 +1800,7 @@ class Clouderizer(object):
 							else:
 								print("Error deploying your project, deploy again")
 								createEvent("deploy failed",creds['cid'],auth_token,servingproject["id"],servingproject["name"])
-								self.delete(servingproject["name"])
+								# self.delete(servingproject["name"])
 								return
 						else:
 							self.delete(servingproject["name"])
@@ -1645,6 +1820,336 @@ class Clouderizer(object):
 			else:
 				print(servingproject)
 
+	# def dlambda(self,*args,**kwargs):
+	# 	lambda_commands = ['deploy','delete']
+
+	# 	if len(args)==0:
+	# 		print("No argument given")
+	# 		return
+
+	# 	if args[0] not in lambda_commands:
+	# 		print("lambda command not supported, available commands : ",end = " ")
+	# 		for i in lambda_commands:
+	# 			print(i, end=" ")
+	# 		return
+
+	# 	if args[0]=="deploy":
+	# 		if len(args)==1:
+	# 			print("No file name given")
+	# 			return
+
+	# 		file_path = args[1]
+
+	# 		model_type=None
+	# 		if "m" in kwargs:
+	# 			model_type = kwargs["m"]
+	# 		if "model_type" in kwargs:
+	# 			model_type = kwargs["model_type"]
+
+	# 		self.ldeploy(file_path,model_type)
+
+
+	def ldeploy(self,file_path,model_type=None,notebook_type=None,requirements=None,predict=None,preprocess=None,postprocess=None,infra="standard",schema=None):
+
+		try:
+			shutil.rmtree(LAMBDA_FUNCTION_DIR)
+		except Exception as e:
+			pass
+		check_aws_cli_exists()
+
+		check_sam_cli_exists()
+
+		if not aws_region_set(LOCAL_DIR):
+			print("aws default region not set : aws configure set default.region {AWS_REGION}")
+			return
+
+		if infra not in LAMBDA_SUPPORTED_INFRA_TYPES:
+			print("Given infra type is not supported, options : ", end=" ")
+			for i in LAMBDA_SUPPORTED_INFRA_TYPES:
+				print(i,end=" ")
+
+		requirements_path = requirements
+
+		creds=self.loggedin()
+		auth_token=creds["token"]
+
+		execution_type="notebook"
+		subtype="pythonscore"
+
+		if model_type and notebook_type:
+			print("Parameters unclear, both notebook and model types selected")
+			return
+
+		if model_type:
+			if model_type in SUPPORTED_MODEL_EXECUTION_TYPES:
+				execution_type="model"
+				if model_type=="python":
+					subtype="pythonscore"
+				else:
+					subtype=model_type
+			else:
+				print("Specified model type is not currently supported "+model_type)
+				print("Supported model types:")
+				for supported_model_type in SUPPORTED_MODEL_EXECUTION_TYPES:
+					print(supported_model_type)
+				return
+
+		if notebook_type:
+			if notebook_type in SUPPORTED_NOTEBOOK_EXECUTION_TYPES:
+				if notebook_type=="python":
+					subtype="pythonscore"
+				else:
+					subtype=notebook_type
+			else:
+				print("Specified notebook type is not currently supported")
+				for supported_notebook_type in SUPPORTED_NOTEBOOK_EXECUTION_TYPES:
+					print(supported_notebook_type)
+				return
+
+		if not file_path:
+			self.help()
+			return
+
+		if not os.path.isfile(file_path):
+			print("Model path not given")
+			return
+
+		# if model_type not in SUPPORTED_MODEL_EXECUTION_TYPES:
+		# 	print("Model type not currently supported")
+		# 	exit(0)
+
+		filename = file_path.split("/")[-1]
+		projectName = unique_project_name(filename)
+		# subtype=model_type
+		pipPackages=[]
+
+		if execution_type=="notebook":
+			notebook_path=file_path
+
+			checkPathExists(notebook_path)
+			# if not os.path.exists(notebook_path):
+			# 	print("Notebook path does not exist "+ notebook_path)
+			# 	return
+
+			if not notebook_path.endswith(".ipynb"):
+				print("File path does not point to a jupyter notebook" + notebook_path)
+				return
+
+			checkPathExists(requirements_path)
+			# if (requirements_path!=None and not os.path.exists(requirements_path)):
+			# 	print("Notebook path does not exist "+ requirements_path)
+			# 	return
+			
+			if requirements_path!=None:
+				pipPackages=parse_requirements(requirements_path)
+			else:
+				pipPackages=get_from_pipreqsnb(notebook_path)
+
+			print("TYPE : NOTEBOOK")
+			print("NOTEBOOK FILE PATH : " + notebook_path)
+			print("FETCHED PIP PACKAGES : ", pipPackages)
+
+			print("Please check the above project attributes.")
+			display_timer("In case something looks wrong, press Ctrl + C and try again with correct values. Deployment will continue in ",10)
+
+			servingproject=create_serving_project(notebook_path.split("/")[-1],creds['cid'],auth_token, projectName=projectName)			
+			if type(servingproject)==dict and "id" in servingproject:
+				update_local_project_list(servingproject["name"],"add")
+				# we will take filename from path
+				file_name=notebook_path.split("/")[-1]
+				s3Path=servingproject["company"]+"/"+servingproject["name"]+"/"+file_name
+				uploadmodel_resp=upload_to_showcase(s3Path,notebook_path,servingproject["company"],auth_token)
+				if uploadmodel_resp=="Success":
+					try:
+						servingmodel=create_serving_model(file_name,s3Path,servingproject,servingproject["id"],pipPackages,auth_token,"true","regular",subtype,"standard")
+					except Exception as e:
+						print(e)
+						print("Some exception occurred while preparing your notebook, please try again!")
+						return
+
+					publishproject=invoke_publishproject(servingproject["id"],infra,auth_token,False)
+
+					if type(servingmodel)==dict and "id" in servingmodel:
+						print("Project created!")
+
+						projectConfig = fetchprojectconfig(servingproject["id"])
+
+						notebookAccessUrl = lambda_deploy(LOCAL_DIR,"nb", projectName, projectName+"-stack", projectConfig, notebook_path,infra,pipPackages,timeout=15*60)
+						update_serving_model(servingmodel['id'],{"status":"Running","status_message":"Deployed successfully","accessUrl":notebookAccessUrl},auth_token)
+					else:
+						print("Notebook upload failed, could not deploy project")
+						return
+
+		
+		if execution_type=="model":
+			model_path=file_path
+
+			if subtype in ["python","onnx","pythonscore"]:
+				predict_path=predict
+			else:
+				predict_path=None
+			preprocess_path=preprocess
+			postprocess_path=postprocess
+
+			inputAttrs={"inputList":[], "rawinputList":[]} 
+			outputAttrs={"outputlist": [],"finaloutput":[]}
+			if schema:
+				inputAttrs, outputAttrs = parse_yaml(schema)
+
+			if subtype in ["python","onnx","pythonscore"] and not predict_path:
+				print("Predict function path not found")
+				return
+
+			if not model_path:
+				print("Model not provided, exiting")
+				return
+
+			checkPathExists(model_path)
+			# if not os.path.exists(model_path):
+			# 	print("Model path does not exist "+ model_path)
+			# 	return
+
+			if preprocess_path:
+				checkPathExists(preprocess_path)
+				checkFileSyntax(preprocess_path)
+				# if not os.path.exists(preprocess_path):
+				# 	print("Preprocess path does not exist " + preprocess_path)
+				# 	return
+				
+				checkFileType(preprocess_path,".py")
+				# if not preprocess_path.endswith(".py"):
+				# 	print("Preprocess path not a python file "+preprocess_path)
+				# 	return
+
+			if postprocess_path: 
+				checkPathExists(postprocess_path)
+				checkFileSyntax(postprocess_path)
+				# if not os.path.exists(postprocess_path):
+				# 	print("Preprocess path does not exist " + postprocess_path)
+				# 	return
+				checkFileType(postprocess_path,".py")
+				# if not postprocess_path.endswith(".py"):
+				# 	print("Postprocess path not a python file "+postprocess_path)
+				# 	return
+
+			if predict_path:
+				checkPathExists(predict_path)
+				checkFileSyntax(predict_path)
+
+				prepend_line(predict_path,"modelPath='/home/app/function/model.file'")
+				# if not os.path.exists(predict_path):
+				# 	print("Preprocess path does not exist " + predict_path)
+				# 	return	
+				checkFileType(predict_path,".py")
+				if not predict_path.endswith(".py"):
+					print("Postprocess path not a python file "+predict_path)
+					return
+			
+			if requirements_path!=None:
+				pipPackages=parse_requirements(requirements_path)
+			else:
+				if subtype in ["python","onnx","pythonscore"] and predict_path:
+					pipPackages=get_from_pipreqsnb(predict_path)
+			print("TYPE : MODEL")
+			print("MODEL PATH : " + model_path)
+			if subtype in ["python","onnx","pythonscore"]:
+				print("PREDICT PATH : " ,predict_path)
+			print("SUBTYPE:" ,subtype)
+			print("PREPROCESS PATH : " , preprocess_path)
+			print("POSTPROCESS PATH : " ,postprocess_path)
+			if subtype in ["python","onnx","pythonscore"]:
+				print("FETCHED PIP PACKAGES : ", pipPackages)
+
+			print("Please check the above project attributes.")
+			display_timer("In case something looks wrong, press Ctrl + C and try again with correct values. Deployment will continue in ",10)
+		
+			servingproject=create_serving_project(file_path.split("/")[-1],creds['cid'],auth_token,projectName=projectName,inputAttrs=inputAttrs,outputAttrs=outputAttrs)
+			# print(type(servingproject))
+			model_path=file_path
+			# preprocess_path=postprocess_path=predict_path=None
+			if type(servingproject)==dict and "id" in servingproject:		
+				update_local_project_list(servingproject["name"],"add")
+				# we will take filename from path
+				file_name=model_path.split("/")[-1]
+				s3Path=servingproject["company"]+"/"+servingproject["name"]+"/"+file_name
+				uploadmodel_resp=upload_to_showcase(s3Path,model_path,servingproject["company"],auth_token)
+				preprocessS3Path=None
+				postprocessS3Path=None
+				predictS3Path=None
+				if uploadmodel_resp=="Success":
+					if preprocess_path:
+						print("Preparing preprocess file for upload")
+						preprocessS3Path=servingproject["company"]+"/"+servingproject["name"]+"/"+preprocess_path.split("/")[-1]
+						uploadpreprocess_resp=upload_to_showcase(preprocessS3Path,preprocess_path,servingproject["company"],auth_token)
+
+						if not uploadpreprocess_resp=="Success":
+							print("Preprocess file upload failed, try again")
+							self.delete(servingproject["name"])
+							return
+
+					if predict_path:
+						print("Preparing predict file for upload")
+						predictS3Path=servingproject["company"]+"/"+servingproject["name"]+"/"+predict_path.split("/")[-1]
+						uploadpredict_resp=upload_to_showcase(predictS3Path,predict_path,servingproject["company"],auth_token)
+
+						if not uploadpredict_resp=="Success":
+							print("Predict file upload failed, try again")
+							self.delete(servingproject["name"])
+							return
+
+					if postprocess_path:
+						print("Preparing postprocess file for upload")
+						postprocessS3Path=servingproject["company"]+"/"+servingproject["name"]+"/"+postprocess_path.split("/")[-1]
+						uploadpostprocess_resp=upload_to_showcase(postprocessS3Path,postprocess_path,servingproject["company"],auth_token)
+
+						if not uploadpostprocess_resp=="Success":
+							print("postprocess file upload failed, try again")
+							self.delete(servingproject["name"])
+							return
+
+					try:
+						deployment_model_type="automl"
+						if subtype in ["python","onnx","pythonscore"]:
+							deployment_model_type="regular"
+						servingmodel=create_serving_model(file_name,s3Path,servingproject,servingproject["id"],pipPackages,auth_token,"false",deployment_model_type,subtype,"standard",preprocess=preprocess_path,postprocess=postprocess_path,predict=predict_path,inputAttrs=inputAttrs,outputAttrs=outputAttrs)
+					except Exception as e:
+						print(e)
+						print("Some exception occurred while preparing your model, please try again!")						
+						return
+
+					publishproject=invoke_publishproject(servingproject["id"],infra,auth_token,False)					
+
+					if type(servingmodel)==dict and "id" in servingmodel:    
+						print("Project created!")
+						if subtype not in ["python","pythonscore"]:
+							print("Fetching I/O attributes from the project")
+							if not parser(model_path,servingproject,servingmodel["id"],subtype,auth_token):
+								print("Some error occurred while fetching attributes")
+								self.delete(servingproject["name"])
+								return
+					else:
+						print("Some error occurred while creating model")
+						return
+
+					projectConfig = fetchprojectconfig(servingproject["id"])
+
+					modelAccessUrl = lambda_deploy(LOCAL_DIR,"model", projectName, projectName+"-stack", projectConfig, model_path,infra,pipPackages,preprocess_path,postprocess_path,predict_path)
+					update_serving_model(servingmodel['id'],{"status":"Running","status_message":"Deployed successfully","accessUrl":modelAccessUrl},auth_token)
+				else:
+					print("Model upload failed, could not deploy project")
+					return
+	
+	def ldelete(self,name):
+		print("Deleting project in AWS Lambda")
+
+		name = find_full_project_name(name)
+
+		print("PROJECT:",name)
+
+		lambda_delete(name,LOCAL_DIR)	
+
+		self.delete(name)	
+
 def main():
 	if len(sys.argv)==1:
 		print("NAME")
@@ -1681,35 +2186,4 @@ def main():
 
 
 if __name__ == '__main__':
-	# if len(sys.argv)==1:
-	# 	print("NAME")
-	# 	print("    cldz")
-	# 	print(" ")
-	# 	print("Options in {} are optional")
-	# 	print(" ")
-	# 	print("SYNOPSIS")
-	# 	print("    cldz COMMAND")
-	# 	print(" ")
-	# 	print("COMMANDS")
-	# 	print("    COMMAND is one of the following:")
-	# 	print("    ")
-	# 	print("        delete $PROJECT_NAME : delete an existing project")
-	# 	print("    ")
-	# 	print("        deploy $NOTEBOOK_PATH {REQUIREMENTS_PATH} : create a new project, upload notebook/model and deploy")
-	# 	print("    ")
-	# 	print("        describe $PROJECT_NAME : get details of the existing project")
-	# 	print("    ")
-	# 	print("        loggedin : to check whether logged in with cldz or not")
-	# 	print("    ")
-	# 	print("        login")
-	# 	print("    ")
-	# 	print("        logout")
-	# 	print("    ")
-	# 	print("        ls : list all the projects")
-	# 	print("    ")
-	# 	print("        start $PROJECT_NAME : start an existing project")
-	# 	print("    ")
-	# 	print("        stop $PROJECT_NAME : stop an existing project")
-	# 	print("    ")
-	# else:
 	main()
